﻿using System;
using System.Windows;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace Wpf_TextStyleSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // Note: the map unit.
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            wpfMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.SimpleColors.LightBlue);

            // Note: the textColumnName, "CNTRY_NAME" for the test shapefiles 'Countries02.shp'.
            TextStyle textStyle = GetTextStyleByUI("CNTRY_NAME", new GeoFont("Arial", 8, DrawingFontStyles.Bold), new GeoSolidBrush(GeoColors.ControlText));

            ShapeFileFeatureLayer shapeFileFeatureLayer = new ShapeFileFeatureLayer(@"../../data/Countries02.shp");
            shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = textStyle;
            shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay layerOverlay = new LayerOverlay();

            //// The tile size of layerOverlay. 
            //layerOverlay.TileWidth = 
            //layerOverlay.TileHeight = 

            layerOverlay.Layers.Add(shapeFileFeatureLayer);

            InMemoryFeatureLayer inMemoryFeatureLayer = new InMemoryFeatureLayer();
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.Highway1;
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Park1;
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            layerOverlay.Layers.Add("inMemoryFeatureLayer", inMemoryFeatureLayer);

            wpfMap1.Overlays.Add(layerOverlay);

            shapeFileFeatureLayer.Open();
            wpfMap1.CurrentExtent = shapeFileFeatureLayer.GetBoundingBox();

            shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.LocalRoad1;
            shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
        }

        private void wpfMap1_MapClick(object sender, MapClickWpfMapEventArgs e)
        {
            var featureLayer = (((LayerOverlay)wpfMap1.Overlays[0]).Layers[0]) as ShapeFileFeatureLayer;
            featureLayer.Open();
            var features = featureLayer.QueryTools.GetFeaturesNearestTo(e.WorldLocation, wpfMap1.MapUnit, 1, ReturningColumnsType.AllColumns);
            var inMemoryFeatureLayer = (((LayerOverlay)wpfMap1.Overlays[0]).Layers[1]) as InMemoryFeatureLayer;
            inMemoryFeatureLayer.InternalFeatures.Clear();
            foreach (var feature in features)
            {
                inMemoryFeatureLayer.InternalFeatures.Add(feature);
            }
            wpfMap1.Refresh();
        }


        private TextStyle GetTextStyleByUI(string textColumnName, GeoFont textFont, GeoSolidBrush textSolidBrush)
        {
            TextStyle textStyle = new TextStyle(textColumnName, textFont, textSolidBrush);

            textStyle.GridSize = int.Parse(textBoxGridSize.Text);

            textStyle.SuppressPartialLabels = (bool)radioButtonSuppressPartialLabelsTrue.IsChecked ? true : false;

            textStyle.OverlappingRule = (bool)radioButtonLabelOverlappingRuleNo.IsChecked ? LabelOverlappingRule.NoOverlapping : LabelOverlappingRule.AllowOverlapping;


            if (radioButtonLabelDuplicateRuleOne.IsChecked == true)
            {
                textStyle.DuplicateRule = LabelDuplicateRule.OneDuplicateLabelPerQuadrant;
            }
            else if (radioButtonLabelDuplicateRuleNo.IsChecked == true)
            {
                textStyle.DuplicateRule = LabelDuplicateRule.NoDuplicateLabels;
            }
            else
            {
                textStyle.DuplicateRule = LabelDuplicateRule.UnlimitedDuplicateLabels;
            }

            textStyle.LabelAllLineParts = (bool)radioButtonLabelAllLinePartsTrue.IsChecked ? true : false;

            if (radioButtonDrawingLevelLabel.IsChecked == true)
            {
                textStyle.DrawingLevel = DrawingLevel.LabelLevel;
            }
            else
            {
                if (radioButtonDrawingLevelOne.IsChecked == true)
                {
                    textStyle.DrawingLevel = DrawingLevel.LevelOne;
                }
                else if (radioButtonDrawingLevelTwo.IsChecked == true)
                {
                    textStyle.DrawingLevel = DrawingLevel.LevelTwo;
                }
                else if (radioButtonDrawingLevelThree.IsChecked == true)
                {
                    textStyle.DrawingLevel = DrawingLevel.LevelThree;
                }
                else if (radioButtonDrawingLevelFour.IsChecked == true)
                {
                    textStyle.DrawingLevel = DrawingLevel.LevelFour;
                }
            }

            if (radioButtonSplineTypeDefault.IsChecked == true)
            {
                textStyle.SplineType = SplineType.Default;
            }
            else if (radioButtonSplineTypeNone.IsChecked == true)
            {
                textStyle.SplineType = SplineType.None;
            }
            else if (radioButtonSplineTypeStandard.IsChecked == true)
            {
                textStyle.SplineType = SplineType.StandardSplining;
            }
            else if (radioButtonSplineTypeForce.IsChecked == true)
            {
                textStyle.SplineType = SplineType.ForceSplining;
            }

            textStyle.TextLineSegmentRatio = double.Parse(textBoxTextLineSegmentRatio.Text);

            textStyle.FittingLineInScreen = (bool)radioButtonFittingLineInScreenTrue.IsChecked ? true : false;

            textStyle.MaxNudgingInPixel = int.Parse(textBoxMaxNudgingInPixel.Text);

            textStyle.NudgingIntervalInPixel = float.Parse(textBoxNudgingIntervalInPixel.Text);

            textStyle.ForceHorizontalLabelForLine = (bool)radioButtonForceHorizontalLabelForLineTrue.IsChecked ? true : false;

            textStyle.AllowLineCarriage = (bool)radioButtonAllowLineCarriage.IsChecked ? true : false;

            textStyle.BestPlacement = (bool)radioButtonBestPlacement.IsChecked ? true : false;

            textStyle.FittingPolygon = (bool)radioButtonFittingPolygon.IsChecked ? true : false;

            textStyle.FittingPolygonFactor = double.Parse(FittingPolygonFactor.Text);

            textStyle.FittingPolygonInScreen = (bool)radioButtonFittingPolygonInScreen.IsChecked ? true : false;

            textStyle.ForceLineCarriage = (bool)radioButtonForceLineCarriage.IsChecked ? true : false;

            textStyle.LabelAllPolygonParts = (bool)radioButtonLabelAllPolygonParts.IsChecked ? true : false;

            if (radioButtonPolygonLabelingLocationModeDefault.IsChecked == true)
            {
                textStyle.PolygonLabelingLocationMode = PolygonLabelingLocationMode.Default;
            }
            else if (radioButtonPolygonLabelingLocationModeCentroid.IsChecked == true)
            {
                textStyle.PolygonLabelingLocationMode = PolygonLabelingLocationMode.Centroid;
            }
            else if (radioButtonPolygonLabelingLocationModeBoundingBoxCenter.IsChecked == true)
            {
                textStyle.PolygonLabelingLocationMode = PolygonLabelingLocationMode.BoundingBoxCenter;
            }

            if (radioButtonPointPlacementUpperLeft.IsChecked == true)
            {
                textStyle.PointPlacement = PointPlacement.UpperLeft;
            }
            else if (radioButtonPointPlacementUpperCenter.IsChecked == true)
            {
                textStyle.PointPlacement = PointPlacement.UpperCenter;
            }
            else if (radioButtonPointPlacementUpperRight.IsChecked == true)
            {
                textStyle.PointPlacement = PointPlacement.UpperRight;
            }
            else if (radioButtonPointPlacementCenterRight.IsChecked == true)
            {
                textStyle.PointPlacement = PointPlacement.CenterRight;
            }
            else if (radioButtonPointPlacementCenter.IsChecked == true)
            {
                textStyle.PointPlacement = PointPlacement.Center;
            }
            else if (radioButtonPointPlacementLowerLeft.IsChecked == true)
            {
                textStyle.PointPlacement = PointPlacement.LowerLeft;
            }
            else if (radioButtonPointPlacementLowerCenter.IsChecked == true)
            {
                textStyle.PointPlacement = PointPlacement.LowerCenter;
            }
            else if (radioButtonPointPlacementCenterLeft.IsChecked == true)
            {
                textStyle.PointPlacement = PointPlacement.CenterLeft;
            }
            else if (radioButtonPointPlacementLowerRight.IsChecked == true)
            {
                textStyle.PointPlacement = PointPlacement.LowerRight;
            }

            return textStyle;
        }

        private void RefreshMap()
        {
            if (wpfMap1.Overlays.Count > 0)
            {
                TextStyle textStyle = GetTextStyleByUI("CNTRY_NAME", new GeoFont("Arial", 8, DrawingFontStyles.Bold), new GeoSolidBrush(GeoColors.ControlText));
                var shapeFileFeatureLayer = (((LayerOverlay)wpfMap1.Overlays[0]).Layers[0]) as ShapeFileFeatureLayer;
                shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = textStyle;
                shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.LocalRoad1;
                shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                wpfMap1.Refresh();
            }
        }

        #region TextStyle Properties 

        private void buttonReGridSize_Click(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonSuppressPartialLabelsTrue_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton1_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonLabelOverlappingRuleNo_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton3_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonLabelDuplicateRuleOne_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonLabelDuplicateRuleNo_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonLabelDuplicateRuleUnlimited_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonLabelAllLinePartsTrue_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton8_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonDrawingLevelOne_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonDrawingLevelTwo_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonDrawingLevelThree_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonDrawingLevelFour_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonDrawingLevelLabel_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonSplineTypeDefault_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonSplineTypeNone_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonSplineTypeStandard_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonSplineTypeForce_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void buttonReTextLineSegmentRatio_Click(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonFittingLineInScreenTrue_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton19_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void buttonReMaxNudgingInPixel_Click(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void buttonReNudgingIntervalInPixel_Click(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonForceHorizontalLabelForLineTrue_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton21_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonAllowLineCarriage_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton2_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonBestPlacement_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton4_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonFittingPolygon_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton5_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void buttonFittingPolygonFactor_Click(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonFittingPolygonInScreen_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton6_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonForceHorizontalLabelForLine_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton7_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonForceLineCarriage_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton9_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonLabelAllLineParts_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton10_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonLabelAllPolygonParts_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButton11_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonPolygonLabelingLocationModeDefault_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonPolygonLabelingLocationModeCentroid_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonPolygonLabelingLocationModeBoundingBoxCenter_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonPointPlacementUpperLeft_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonPointPlacementUpperCenter_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonPointPlacementUpperRight_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonPointPlacementCenterRight_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonPointPlacementCenter_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonPointPlacementLowerLeft_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonPointPlacementLowerCenter_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonPointPlacementCenterLeft_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }

        private void radioButtonPointPlacementLowerRight_Checked(object sender, RoutedEventArgs e)
        {
            RefreshMap();
        }
        #endregion
    }
}
